@@ -2390,10 +2390,7 @@ gcm_prefs_window_realize_cb (GtkWidget *widget, CcColorPanel *prefs)
 static const char *
 cc_color_panel_get_help_uri (CcPanel *panel)
 {
-  if (!g_strcmp0(g_getenv("XDG_CURRENT_DESKTOP"), "Unity"))
-    return "help:ubuntu-help/color";
-  else
-    return "help:gnome-help/color";
+  return "help:ubuntu-help/color";
 }
 
 static void
@@ -2675,12 +2672,6 @@ cc_color_panel_init (CcColorPanel *prefs)
   g_signal_connect (widget, "realize",
                     G_CALLBACK (gcm_prefs_window_realize_cb),
                     prefs);
-
-  widget = WID (priv->builder, "linkbutton_help");
-  if (!g_strcmp0(g_getenv("XDG_CURRENT_DESKTOP"), "Unity"))
-    g_object_set (G_OBJECT (widget),
-              "uri", "help:ubuntu-help/color-whyimportant",
-              NULL);
 }
 
 void
