@@ -40,7 +40,7 @@
 #include <GL/gl.h>
 #include <GL/glx.h>
 
-#include <webkit/webkit.h>
+#include <webkit2/webkit2.h>
 
 #include "hostname-helper.h"
 #include "gsd-disk-space-helper.h"
@@ -450,10 +450,10 @@ get_graphics_data (void)
   result->glx_renderer = get_graphics_data_glx_renderer ();
   result->xorg_vesa_hardware = get_graphics_data_xorg_vesa_hardware ();
 
-  if (result->xorg_vesa_hardware != NULL)
-    result->hardware_string = result->xorg_vesa_hardware;
-  else if (result->glx_renderer != NULL)
+  if (result->glx_renderer != NULL)
     result->hardware_string = result->glx_renderer;
+  else if (result->xorg_vesa_hardware != NULL)
+    result->hardware_string = result->xorg_vesa_hardware;
   else
     result->hardware_string = _("Unknown");
 
@@ -579,14 +579,38 @@ get_primary_disc_info_start (CcInfoPanel *self)
   g_object_unref (file);
 }
 
+static GList *
+convert_points_to_entries (GList *points)
+{
+  GList *entries = NULL;
+  GList *p;
+
+  for (p = points; p != NULL; p = p->next)
+    {
+      GUnixMountPoint *point = p->data;
+      GUnixMountEntry *mount;
+      const gchar *mount_path = g_unix_mount_point_get_mount_path (point);
+
+      mount = g_unix_mount_at (mount_path, NULL);
+      if (mount)
+        entries = g_list_append (entries, mount);
+    }
+
+  return entries;
+}
+
 static void
 get_primary_disc_info (CcInfoPanel *self)
 {
-  GList        *points;
-  GList        *p;
+  GList *points, *entries = NULL;
+  GList *p;
 
   points = g_unix_mount_points_get (NULL);
-  for (p = points; p != NULL; p = p->next)
+
+  entries = convert_points_to_entries (points);
+  g_list_free_full (points, (GDestroyNotify) g_unix_mount_point_free);
+
+  for (p = entries; p != NULL; p = p->next)
     {
       GUnixMountEntry *mount = p->data;
       const char *mount_path;
@@ -604,7 +628,7 @@ get_primary_disc_info (CcInfoPanel *self)
 
       self->priv->primary_mounts = g_list_prepend (self->priv->primary_mounts, mount);
     }
-  g_list_free (points);
+  g_list_free (entries);
 
   get_primary_disc_info_start (self);
 }
@@ -733,38 +757,51 @@ on_section_changed (GtkTreeSelection  *selection,
   gtk_tree_path_free (path);
 }
 
-static gboolean            
-url_nav_callback (WebKitWebView             *web_view,
-              WebKitWebFrame            *frame,
-              WebKitNetworkRequest      *request,
-              WebKitWebNavigationAction *navigation_action,
-              WebKitWebPolicyDecision   *decision,
-              gpointer                   user_data)
+static gboolean
+on_decide_policy (WebKitWebView             *web_view,
+                  WebKitPolicyDecision      *decision,
+                  WebKitPolicyDecisionType   decision_type,
+                  gpointer                   user_data)
 {
-    gtk_show_uri (gtk_widget_get_screen (user_data),
-                  webkit_network_request_get_uri (request),
-                  GDK_CURRENT_TIME, NULL);
-    return TRUE;
+    WebKitNavigationPolicyDecision *navigation_decision;
+    WebKitNavigationAction *navigation_action;
+    WebKitNavigationType navigation_type;
+    WebKitURIRequest *request;
+    const gchar *uri;
+
+    switch (decision_type)
+    {
+        case WEBKIT_POLICY_DECISION_TYPE_NAVIGATION_ACTION:
+            navigation_decision = WEBKIT_NAVIGATION_POLICY_DECISION(decision);
+            navigation_action = webkit_navigation_policy_decision_get_navigation_action (navigation_decision);
+            request = webkit_navigation_action_get_request (navigation_action);
+            navigation_type = webkit_navigation_action_get_navigation_type (navigation_action);
+
+            if (navigation_type == WEBKIT_NAVIGATION_TYPE_LINK_CLICKED)
+            {
+                uri = webkit_uri_request_get_uri (request);
+
+                webkit_policy_decision_ignore (decision);
+
+                gtk_show_uri (gtk_widget_get_screen (GTK_WIDGET (web_view)),
+                              uri, GDK_CURRENT_TIME, NULL);
+                return TRUE;
+            }
+        default:
+            return FALSE;
+    }
 }
 
 static void
 info_panel_setup_notice (CcInfoPanel  *self)
 {
-  GtkWidget *sw;
-  
-  sw = gtk_scrolled_window_new (NULL, NULL);
-  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw),
-                                  GTK_POLICY_AUTOMATIC,
-                                  GTK_POLICY_AUTOMATIC);
-
-  WebKitWebView *webView = WEBKIT_WEB_VIEW(webkit_web_view_new());
+  WebKitWebView *webView = WEBKIT_WEB_VIEW (webkit_web_view_new());
   webkit_web_view_load_uri(webView, "file:///usr/share/unity-control-center/searchingthedashlegalnotice.html");
-  g_signal_connect (G_OBJECT (webView), "navigation-policy-decision-requested",
-                    G_CALLBACK (url_nav_callback), sw);
+  g_signal_connect (G_OBJECT (webView), "decide-policy",
+                    G_CALLBACK (on_decide_policy), NULL);
 
-  gtk_container_add (GTK_CONTAINER (sw), GTK_WIDGET (webView));
-  gtk_notebook_append_page (GTK_NOTEBOOK (WID ("notebook")), sw, NULL);
-  gtk_widget_show_all(sw);
+  gtk_notebook_append_page (GTK_NOTEBOOK (WID ("notebook")), GTK_WIDGET (webView), NULL);
+  gtk_widget_show_all (GTK_WIDGET (webView));
 }
 
 static void
@@ -800,7 +837,7 @@ default_app_changed (GtkAppChooserButton *button,
       pattern = g_pattern_spec_new (app_data->extra_type_filter);
       mime_types = g_app_info_get_supported_types (info);
 
-      for (i = 0; mime_types[i]; i++)
+      for (i = 0; mime_types && mime_types[i]; i++)
         {
           if (!g_pattern_match_string (pattern, mime_types[i]))
             continue;
@@ -1402,13 +1439,10 @@ info_panel_setup_selector (CcInfoPanel  *self)
                       _("Removable Media"),
                       -1);
 
-  if (!g_strcmp0 (g_getenv ("XDG_CURRENT_DESKTOP"), "Unity"))
-    {
-      gtk_list_store_append (model, &iter);
-      gtk_list_store_set (model, &iter, section_name_column,
-                          _("Legal Notice"),
-                          -1);
-    }
+  gtk_list_store_append (model, &iter);
+  gtk_list_store_set (model, &iter, section_name_column,
+                      _("Legal Notice"),
+                      -1);
 
   g_signal_connect (selection, "changed",
                     G_CALLBACK (on_section_changed), self);
@@ -1934,8 +1968,7 @@ cc_info_panel_init (CcInfoPanel *self)
   info_panel_setup_overview (self);
   info_panel_setup_default_apps (self);
   info_panel_setup_media (self);
-  if (!g_strcmp0 (g_getenv ("XDG_CURRENT_DESKTOP"), "Unity"))
-    info_panel_setup_notice (self);
+  info_panel_setup_notice (self);
 }
 
 void
