@@ -26,8 +26,8 @@
 #include "keyboard-shortcuts.h"
 #include "wm-common.h"
 
-#define BINDINGS_SCHEMA "org.gnome.settings-daemon.plugins.media-keys"
-#define CUSTOM_KEYS_BASENAME "/org/gnome/settings-daemon/plugins/media-keys/custom-keybindings"
+#define BINDINGS_SCHEMA "com.canonical.unity.settings-daemon.plugins.media-keys"
+#define CUSTOM_KEYS_BASENAME "/com/canonical/unity/settings-daemon/plugins/media-keys/custom-keybindings"
 #define CUSTOM_SHORTCUTS_ID "custom"
 
 typedef struct {
