@@ -249,11 +249,11 @@ accept_password_dialog (GtkButton        *button,
                                 gdk_window_set_cursor (gtk_widget_get_window (um->dialog), cursor);
                                 gdk_display_flush (display);
                                 g_object_unref (cursor);
-                                return;
+                        } else {
+                                act_user_set_password (um->user, password, hint);
                         }
 
                         act_user_set_password_mode (um->user, ACT_USER_PASSWORD_MODE_REGULAR);
-                        act_user_set_password (um->user, password, hint);
                         break;
 
                 case UM_PASSWORD_DIALOG_MODE_SET_AT_LOGIN:
@@ -268,6 +268,10 @@ accept_password_dialog (GtkButton        *button,
                 case UM_PASSWORD_DIALOG_MODE_LOCK_ACCOUNT:
                         act_user_set_locked (um->user, TRUE);
                         act_user_set_automatic_login (um->user, FALSE);
+
+                        /*Set the password mode to regular in case the user is currently in the nopasswdlogin group.
+                        Otherwise, the user will still be able to login even though the account is disabled. */
+                        act_user_set_password_mode (um->user, ACT_USER_PASSWORD_MODE_REGULAR);
                         break;
 
                 case UM_PASSWORD_DIALOG_MODE_UNLOCK_ACCOUNT:
@@ -617,14 +621,9 @@ um_password_dialog_new (void)
 
         widget = (GtkWidget *) gtk_builder_get_object (builder, "password-normal-strength-hints-label");
         old_label = gtk_label_get_label (GTK_LABEL (widget));
-        if (!g_strcmp0(g_getenv("XDG_CURRENT_DESKTOP"), "Unity"))
-            label = g_strdup_printf ("<a href=\"%s\">%s</a>",
+        label = g_strdup_printf ("<a href=\"%s\">%s</a>",
                                  "help:ubuntu-help/user-goodpassword",
                                   old_label);
-        else
-            label = g_strdup_printf ("<a href=\"%s\">%s</a>",
-                                  "help:gnome-help/user-goodpassword",
-                                  old_label);
         gtk_label_set_markup (GTK_LABEL (widget), label);
         g_free (label);
 
